@@ -191,6 +191,8 @@ struct _YelpLocationEntryPrivate
     gboolean   search_mode;
     guint      pulse;
     gulong bookmarks_changed;
+
+    gboolean   icon_is_clear;
 };
 
 enum {
@@ -214,6 +216,7 @@ enum {
     COMPLETION_COL_TITLE,
     COMPLETION_COL_DESC,
     COMPLETION_COL_ICON,
+    COMPLETION_COL_PIXBUF,
     COMPLETION_COL_PAGE,
     COMPLETION_COL_FLAGS
 };
@@ -370,6 +373,7 @@ static void
 yelp_location_entry_init (YelpLocationEntry *entry)
 {
     YelpLocationEntryPrivate *priv = GET_PRIV (entry);
+
     priv->search_mode = FALSE;
     g_object_set (entry, "entry-text-column", HISTORY_COL_TITLE, NULL);
 }
@@ -404,7 +408,7 @@ location_entry_constructed (GObject *object)
     priv->history = gtk_list_store_new (8,
                                         G_TYPE_STRING,  /* title */
                                         G_TYPE_STRING,  /* desc */
-                                        G_TYPE_STRING,  /* icon */
+                                        G_TYPE_OBJECT,  /* icon */
                                         G_TYPE_STRING,  /* uri */
                                         G_TYPE_STRING,  /* doc */
                                         G_TYPE_STRING,  /* page */
@@ -416,16 +420,19 @@ location_entry_constructed (GObject *object)
                       object);
     g_object_set (object, "model", priv->history, NULL);
     if (priv->enable_search) {
+        GdkPixbuf *pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (),
+                                                           "system-search");
         gtk_list_store_append (priv->history, &iter);
         gtk_list_store_set (priv->history, &iter,
                             HISTORY_COL_FLAGS, LOCATION_ENTRY_IS_SEPARATOR,
                             -1);
         gtk_list_store_append (priv->history, &iter);
         gtk_list_store_set (priv->history, &iter,
-                            HISTORY_COL_ICON, "system-search",
+                            HISTORY_COL_ICON, pixbuf,
                             HISTORY_COL_TITLE, _("Search..."),
                             HISTORY_COL_FLAGS, LOCATION_ENTRY_IS_SEARCH,
                             -1);
+        g_object_unref (pixbuf);
     }
 
     /* Set up the history drop-down */
@@ -448,7 +455,7 @@ location_entry_constructed (GObject *object)
     g_object_set (priv->icon_cell, "yalign", 0.2, NULL);
     gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (object),
                                     priv->icon_cell,
-                                    "icon-name",
+                                    "pixbuf",
                                     HISTORY_COL_ICON,
                                     NULL);
     gtk_cell_layout_reorder (GTK_CELL_LAYOUT (object), priv->icon_cell, 0);
@@ -611,28 +618,46 @@ location_entry_location_selected (YelpLocationEntry *entry)
 static void
 location_entry_search_activated  (YelpLocationEntry *entry)
 {
-    printf ("FIXME: search_activated\n");
+    YelpUri *base, *uri;
+    YelpLocationEntryPrivate *priv = GET_PRIV (entry);
+
+    g_object_get (priv->view, "yelp-uri", &base, NULL);
+    uri = yelp_uri_new_search (base,
+                               gtk_entry_get_text (GTK_ENTRY (priv->text_entry)));
+    g_object_unref (base);
+    yelp_view_load_uri (priv->view, uri);
+    gtk_widget_grab_focus (GTK_WIDGET (priv->view));
 }
 
 static void
 location_entry_bookmark_clicked  (YelpLocationEntry *entry)
 {
     YelpUri *uri;
-    gchar *doc_uri, *page_id, *icon, *title;
+    gchar *doc_uri, *page_id;
     YelpLocationEntryPrivate *priv = GET_PRIV (entry);
 
     g_object_get (priv->view,
                   "yelp-uri", &uri,
                   "page-id", &page_id,
-                  "page-icon", &icon,
-                  "page-title", &title,
                   NULL);
     doc_uri = yelp_uri_get_document_uri (uri);
-    yelp_bookmarks_add_bookmark (priv->bookmarks, doc_uri, page_id, icon, title);
+    if (priv->bookmarks && doc_uri && page_id) {
+        if (!yelp_bookmarks_is_bookmarked (priv->bookmarks, doc_uri, page_id)) {
+            gchar *icon, *title;
+            g_object_get (priv->view,
+                          "page-icon", &icon,
+                          "page-title", &title,
+                          NULL);
+            yelp_bookmarks_add_bookmark (priv->bookmarks, doc_uri, page_id, icon, title);
+            g_free (icon);
+            g_free (title);
+        }
+        else {
+            yelp_bookmarks_remove_bookmark (priv->bookmarks, doc_uri, page_id);
+        }
+    }
     g_free (doc_uri);
     g_free (page_id);
-    g_free (icon);
-    g_free (title);
     g_object_unref (uri);
 }
 
@@ -645,7 +670,15 @@ location_entry_start_search (YelpLocationEntry *entry,
     if (!priv->enable_search)
         return;
     if (clear && !priv->search_mode) {
-        gtk_entry_set_text (GTK_ENTRY (priv->text_entry), "");
+        GtkTreeIter iter;
+        gchar *page;
+        gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->history), &iter);
+        gtk_tree_model_get (GTK_TREE_MODEL (priv->history), &iter,
+                            HISTORY_COL_PAGE, &page,
+                            -1);
+        if (!(page && g_str_has_prefix (page, "search=")))
+            gtk_entry_set_text (GTK_ENTRY (priv->text_entry), "");
+        g_free (page);
     }
     priv->search_mode = TRUE;
     location_entry_set_entry (entry, FALSE);
@@ -674,6 +707,7 @@ location_entry_cancel_search (YelpLocationEntry *entry)
                                   priv->completion);
         g_object_unref (priv->completion);
     }
+    gtk_editable_select_region (GTK_EDITABLE (priv->text_entry), 0, 0);
 }
 
 static void
@@ -709,8 +743,8 @@ location_entry_set_completion (YelpLocationEntry *entry,
     gtk_cell_layout_reorder (GTK_CELL_LAYOUT (priv->completion), icon_cell, 0);
     gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (priv->completion),
                                     icon_cell,
-                                    "icon-name",
-                                    COMPLETION_COL_ICON,
+                                    "pixbuf",
+                                    COMPLETION_COL_PIXBUF,
                                     NULL);
     if (priv->bookmarks) {
         bookmark_cell = gtk_cell_renderer_pixbuf_new ();
@@ -731,20 +765,31 @@ location_entry_set_entry (YelpLocationEntry *entry, gboolean emit)
     GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (entry));
     GtkTreePath *path = NULL;
     GtkTreeIter iter;
-    gchar *icon_name;
+    GdkPixbuf *pixbuf;
 
     if (priv->search_mode) {
-        gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
-                                           GTK_ENTRY_ICON_PRIMARY,
-                                           "system-search");
-        gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
-                                           GTK_ENTRY_ICON_SECONDARY,
-                                           "edit-clear");
+        YelpSettings *settings = yelp_settings_get_default ();
+        GdkPixbuf *pixbuf;
+        pixbuf = yelp_settings_get_icon_pixbuf (settings, "system-search");
+        gtk_entry_set_icon_from_pixbuf (GTK_ENTRY (priv->text_entry),
+                                        GTK_ENTRY_ICON_PRIMARY,
+                                        pixbuf);
+        g_object_unref (pixbuf);
+        pixbuf = yelp_settings_get_icon_pixbuf (settings, "edit-clear");
+        gtk_entry_set_icon_from_pixbuf (GTK_ENTRY (priv->text_entry),
+                                        GTK_ENTRY_ICON_SECONDARY,
+                                        pixbuf);
+        g_object_unref (pixbuf);
         gtk_entry_set_icon_tooltip_text (GTK_ENTRY (priv->text_entry),
                                          GTK_ENTRY_ICON_SECONDARY,
-                                         "Clear the search text");
+                                         _("Clear the search text"));
+        priv->icon_is_clear = TRUE;
         return;
     }
+    else {
+        priv->icon_is_clear = FALSE;
+    }
+
 
     if (priv->row)
         path = gtk_tree_row_reference_get_path (priv->row);
@@ -755,39 +800,57 @@ location_entry_set_entry (YelpLocationEntry *entry, gboolean emit)
         gtk_tree_model_get_iter (model, &iter, path);
         gtk_tree_model_get (model, &iter,
                             HISTORY_COL_TITLE, &text,
-                            HISTORY_COL_ICON, &icon_name,
+                            HISTORY_COL_ICON, &pixbuf,
                             HISTORY_COL_FLAGS, &flags,
                             HISTORY_COL_DOC, &doc_uri,
                             HISTORY_COL_PAGE, &page_id,
                             -1);
         if (flags & LOCATION_ENTRY_IS_LOADING) {
-            gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
-                                               GTK_ENTRY_ICON_PRIMARY,
-                                               "image-loading");
+            GdkPixbuf *loading = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (),
+                                                                "image-loading");
+            gtk_entry_set_icon_from_pixbuf (GTK_ENTRY (priv->text_entry),
+                                            GTK_ENTRY_ICON_PRIMARY,
+                                            loading);
+            g_object_unref (loading);
             if (priv->pulse > 0)
                 g_source_remove (priv->pulse);
             priv->pulse = g_timeout_add (80, location_entry_pulse, entry);
         }
         else {
-            gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
-                                               GTK_ENTRY_ICON_PRIMARY,
-                                               icon_name);
+            gtk_entry_set_icon_from_pixbuf (GTK_ENTRY (priv->text_entry),
+                                            GTK_ENTRY_ICON_PRIMARY,
+                                            pixbuf);
         }
+        g_object_unref (pixbuf);
         if (priv->bookmarks && doc_uri && page_id) {
             if (!yelp_bookmarks_is_bookmarked (priv->bookmarks, doc_uri, page_id)) {
-                gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
-                                                   GTK_ENTRY_ICON_SECONDARY,
-                                                   "bookmark-new");
+                pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (),
+                                                        "yelp-bookmark-add");
+                gtk_entry_set_icon_from_pixbuf (GTK_ENTRY (priv->text_entry),
+                                                GTK_ENTRY_ICON_SECONDARY,
+                                                pixbuf);
+                g_object_unref (pixbuf);
                 gtk_entry_set_icon_tooltip_text (GTK_ENTRY (priv->text_entry),
                                                  GTK_ENTRY_ICON_SECONDARY,
-                                                 "Bookmark this page");
+                                                 _("Bookmark this page"));
             }
             else {
-                gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
-                                                   GTK_ENTRY_ICON_SECONDARY,
-                                                   NULL);
+                pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (),
+                                                        "yelp-bookmark-remove");
+                gtk_entry_set_icon_from_pixbuf (GTK_ENTRY (priv->text_entry),
+                                                GTK_ENTRY_ICON_SECONDARY,
+                                                pixbuf);
+                g_object_unref (pixbuf);
+                gtk_entry_set_icon_tooltip_text (GTK_ENTRY (priv->text_entry),
+                                                 GTK_ENTRY_ICON_SECONDARY,
+                                                 _("Remove bookmark"));
             }
         }
+        else {
+            gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
+                                               GTK_ENTRY_ICON_SECONDARY,
+                                               NULL);
+        }
         g_free (doc_uri);
         g_free (page_id);
         gtk_entry_set_text (GTK_ENTRY (priv->text_entry), text);
@@ -956,13 +1019,10 @@ entry_icon_press_cb (GtkEntry            *gtkentry,
     YelpLocationEntryPrivate *priv = GET_PRIV (entry);
 
     if (icon_pos == GTK_ENTRY_ICON_SECONDARY) {
-        const gchar *name = gtk_entry_get_icon_name (gtkentry, icon_pos);
-        if (g_str_equal (name, "edit-clear")) {
+        if (priv->icon_is_clear)
             location_entry_cancel_search (entry);
-        }
-        else if  (g_str_equal (name, "bookmark-new")) {
+        else
             g_signal_emit (entry, location_entry_signals[BOOKMARK_CLICKED], 0);
-        }
     }
 }
 
@@ -1026,13 +1086,13 @@ cell_set_bookmark_icon (GtkCellLayout     *layout,
     YelpLocationEntryPrivate *priv = GET_PRIV (entry);
 
     if (priv->bookmarks == NULL) {
-        g_object_set (cell, "icon-name", NULL, NULL);
+        g_object_set (cell, "pixbuf", NULL, NULL);
         return;
     }
 
     gtk_tree_model_get (model, iter, HISTORY_COL_FLAGS, &flags, -1);
     if (flags & (LOCATION_ENTRY_IS_SEPARATOR | LOCATION_ENTRY_IS_SEARCH)) {
-        g_object_set (cell, "icon-name", NULL, NULL);
+        g_object_set (cell, "pixbuf", NULL, NULL);
         return;
     }
     
@@ -1041,10 +1101,14 @@ cell_set_bookmark_icon (GtkCellLayout     *layout,
                         HISTORY_COL_PAGE, &page_id,
                         -1);
     if (doc_uri && page_id &&
-        yelp_bookmarks_is_bookmarked (priv->bookmarks, doc_uri, page_id))
-        g_object_set (cell, "icon-name", "bookmark", NULL);
+        yelp_bookmarks_is_bookmarked (priv->bookmarks, doc_uri, page_id)) {
+        GdkPixbuf *pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (),
+                                                           "yelp-bookmark-remove");
+        g_object_set (cell, "pixbuf", pixbuf, NULL);
+        g_object_unref (pixbuf);
+    }
     else
-        g_object_set (cell, "icon-name", NULL, NULL);
+        g_object_set (cell, "pixbuf", NULL, NULL);
 
     g_free (doc_uri);
     g_free (page_id);
@@ -1066,10 +1130,15 @@ cell_set_completion_bookmark_icon (GtkCellLayout     *layout,
                             -1);
 
         if (page_id && yelp_bookmarks_is_bookmarked (priv->bookmarks,
-                                                     priv->completion_uri, page_id))
-            g_object_set (cell, "icon-name", "bookmark", NULL);
-        else
-            g_object_set (cell, "icon-name", NULL, NULL);
+                                                     priv->completion_uri, page_id)) {
+            GdkPixbuf *pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (),
+                                                               "yelp-bookmark-remove");
+            g_object_set (cell, "pixbuf", pixbuf, NULL);
+            g_object_unref (pixbuf);
+        }
+        else {
+            g_object_set (cell, "pixbuf", NULL, NULL);
+        }
 
         g_free (page_id);
     }
@@ -1251,6 +1320,7 @@ view_loaded (YelpView          *view,
                                                      G_TYPE_STRING,  /* title */
                                                      G_TYPE_STRING,  /* desc */
                                                      G_TYPE_STRING,  /* icon */
+                                                     G_TYPE_OBJECT,  /* pixbuf */
                                                      G_TYPE_STRING,  /* uri */
                                                      G_TYPE_INT      /* flags */
                                                      );
@@ -1259,26 +1329,32 @@ view_loaded (YelpView          *view,
                                                      entry_completion_sort,
                                                      NULL, NULL);
             g_hash_table_insert (completions, g_strdup (doc_uri), completion);
-            ids = yelp_document_list_page_ids (document);
-            for (i = 0; ids[i]; i++) {
-                GtkTreeIter iter;
-                gchar *title, *desc, *icon;
-                gtk_list_store_insert (GTK_LIST_STORE (base), &iter, 0);
-                title = yelp_document_get_page_title (document, ids[i]);
-                desc = yelp_document_get_page_desc (document, ids[i]);
-                icon = yelp_document_get_page_icon (document, ids[i]);
-                gtk_list_store_set (base, &iter,
-                                    COMPLETION_COL_TITLE, title,
-                                    COMPLETION_COL_DESC, desc,
-                                    COMPLETION_COL_ICON, icon,
-                                    COMPLETION_COL_PAGE, ids[i],
-                                    -1);
-                g_free (icon);
-                g_free (desc);
-                g_free (title);
+            if (document != NULL) {
+                ids = yelp_document_list_page_ids (document);
+                for (i = 0; ids[i]; i++) {
+                    GtkTreeIter iter;
+                    gchar *title, *desc, *icon;
+                    GdkPixbuf *pixbuf;
+                    gtk_list_store_insert (GTK_LIST_STORE (base), &iter, 0);
+                    title = yelp_document_get_page_title (document, ids[i]);
+                    desc = yelp_document_get_page_desc (document, ids[i]);
+                    icon = yelp_document_get_page_icon (document, ids[i]);
+                    pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (), icon);
+                    gtk_list_store_set (base, &iter,
+                                        COMPLETION_COL_TITLE, title,
+                                        COMPLETION_COL_DESC, desc,
+                                        COMPLETION_COL_ICON, icon,
+                                        COMPLETION_COL_PIXBUF, pixbuf,
+                                        COMPLETION_COL_PAGE, ids[i],
+                                        -1);
+                    g_object_unref (pixbuf);
+                    g_free (icon);
+                    g_free (desc);
+                    g_free (title);
+                }
+                g_strfreev (ids);
             }
             g_object_unref (base);
-            g_strfreev (ids);
         }
         g_free (priv->completion_uri);
         priv->completion_uri = doc_uri;
@@ -1328,13 +1404,17 @@ view_uri_selected (YelpView          *view,
     else {
         gint num;
         GtkTreeIter last;
+        GdkPixbuf *pixbuf;
+        pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (),
+                                                "help-contents");
         gtk_list_store_prepend (priv->history, &iter);
         gtk_list_store_set (priv->history, &iter,
                             HISTORY_COL_TITLE, _("Loading"),
-                            HISTORY_COL_ICON, "help-contents",
+                            HISTORY_COL_ICON, pixbuf,
                             HISTORY_COL_URI, struri,
                             HISTORY_COL_FLAGS, LOCATION_ENTRY_IS_LOADING,
                             -1);
+        g_object_unref (pixbuf);
         /* Limit to 15 entries. There are two extra for the search entry and
          * the separator above it.
          */
@@ -1361,35 +1441,42 @@ view_page_title (YelpView          *view,
                  YelpLocationEntry *entry)
 {
     GtkTreeIter first;
-    gchar *title, *frag;
-    YelpUri *uri;
     YelpLocationEntryPrivate *priv = GET_PRIV (entry);
 
-    g_object_get (view, "page-title", &title, NULL);
-    if (title == NULL)
-        return;
-
-    g_object_get (view, "yelp-uri", &uri, NULL);
-    frag = yelp_uri_get_frag_id (uri);
-
     if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->history), &first)) {
+        gchar *title, *frag = NULL;
+        YelpViewState state;
+        YelpUri *uri;
+
+        g_object_get (view,
+                      "page-title", &title,
+                      "state", &state,
+                      NULL);
+        if (title == NULL)
+            return;
+
+        if (state != YELP_VIEW_STATE_ERROR) {
+            YelpUri *uri;
+            g_object_get (view, "yelp-uri", &uri, NULL);
+            frag = yelp_uri_get_frag_id (uri);
+            g_object_unref (uri);
+        }
+
         if (frag) {
             gchar *tmp = g_strdup_printf ("%s (#%s)", title, frag);
             gtk_list_store_set (priv->history, &first,
                                 HISTORY_COL_TITLE, tmp,
                                 -1);
             g_free (tmp);
+            g_free (frag);     
         }
         else {
             gtk_list_store_set (priv->history, &first,
                                 HISTORY_COL_TITLE, title,
                                 -1);
         }
+        g_free (title);
     }
-
-    g_free (frag);     
-    g_free (title);
-    g_object_unref (uri);
 }
 
 static void
@@ -1419,16 +1506,20 @@ view_page_icon (YelpView          *view,
 {
     GtkTreeIter first;
     gchar *icon;
+    GdkPixbuf *pixbuf;
     YelpLocationEntryPrivate *priv = GET_PRIV (entry);
 
     g_object_get (view, "page-icon", &icon, NULL);
     if (icon == NULL)
         return;
 
+    pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (), icon);
+
     if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->history), &first))
         gtk_list_store_set (priv->history, &first,
-                            HISTORY_COL_ICON, icon,
+                            HISTORY_COL_ICON, pixbuf,
                             -1);
+    g_object_unref (pixbuf);
     g_free (icon);
 }
 
@@ -1452,20 +1543,35 @@ bookmarks_changed (YelpBookmarks      *bookmarks,
                             HISTORY_COL_PAGE, &page_id,
                             -1);
         if (this_uri && g_str_equal (this_uri, doc_uri) && page_id) {
+            GdkPixbuf *pixbuf;
             if (!yelp_bookmarks_is_bookmarked (priv->bookmarks, doc_uri, page_id)) {
-                gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
-                                                   GTK_ENTRY_ICON_SECONDARY,
-                                                   "bookmark-new");
+                pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (),
+                                                        "yelp-bookmark-add");
+                gtk_entry_set_icon_from_pixbuf (GTK_ENTRY (priv->text_entry),
+                                                GTK_ENTRY_ICON_SECONDARY,
+                                                pixbuf);
+                g_object_unref (pixbuf);
                 gtk_entry_set_icon_tooltip_text (GTK_ENTRY (priv->text_entry),
                                                  GTK_ENTRY_ICON_SECONDARY,
-                                                 "Bookmark this page");
+                                                 _("Bookmark this page"));
             }
             else {
-                gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
-                                                   GTK_ENTRY_ICON_SECONDARY,
-                                                   NULL);
+                pixbuf = yelp_settings_get_icon_pixbuf (yelp_settings_get_default (),
+                                                        "yelp-bookmark-remove");
+                gtk_entry_set_icon_from_pixbuf (GTK_ENTRY (priv->text_entry),
+                                                GTK_ENTRY_ICON_SECONDARY,
+                                                pixbuf);
+                g_object_unref (pixbuf);
+                gtk_entry_set_icon_tooltip_text (GTK_ENTRY (priv->text_entry),
+                                                 GTK_ENTRY_ICON_SECONDARY,
+                                                 _("Remove bookmark"));
             }
         }
+        else {
+            gtk_entry_set_icon_from_icon_name (GTK_ENTRY (priv->text_entry),
+                                               GTK_ENTRY_ICON_SECONDARY,
+                                               NULL);
+        }
         g_free (this_uri);
         g_free (page_id);
     }
